module TwoFactorDevices
  # Controller for user 2fa devices management
  class UserDevicesController < ApplicationController
    before_action :check_user
    before_action :skip_for_confirmed, only: %i[confirmation_page resend_confirmation_code]

    layout TwoFactorDevices.layout || 'two_factor_devices'

    def index
      @devices = current_user.two_factor_devices.order(is_primary: :desc, id: :asc)
      @available_types = TwoFactorDevices.available_types

      respond_to do |format|
        format.html
        format.json { render json: { available_types: @available_types, devices: @devices } }
      end
    end

    def new
      @device_type = device_type
      @device = device_manager.prepare_device(current_user, device_type)

      respond_to do |format|
        format.html
        format.json { render json: { device_type: @device_type, device: @device } }
      end
    end

    def create
      @device_type = device_type
      @device = device_manager.prepare_device(current_user, device_type, device_params)
      device_manager.register_device!(@device)
      device_manager.make_challenge(@device)

      redirect_to confirmation_page_device_path(@device)
    rescue TwoFactorDevices::SendOtpError => e
      redirect_to devices_path, flash: { danger: otp_error_message(e) }
    rescue TwoFactorDevices::InvalidDevice
      render :new
    end

    def confirmation_page
      @device = device

      respond_to do |format|
        format.html
        format.json { render json: { device: @device } }
      end
    end

    def confirm
      code = params[:confirmation_code]

      if device_manager.validate(code.to_s)
        device.confirm!
        return redirect_to devices_path
      end

      redirect_to confirmation_page_device_path(device),
                  flash: { danger: t('two_factor_devices.user_devices.failed_confirmation') }
    end

    def resend_confirmation_code
      device_manager.make_challenge(device)

      redirect_to confirmation_page_device_path(device)
    rescue TwoFactorDevices::SendOtpError => e
      redirect_to confirmation_page_device_path(device), flash: { danger: otp_error_message(e) }
    end

    def destroy
      device.destroy

      redirect_to devices_path, flash: { info: t('two_factor_devices.user_devices.success_destroy') }
    end

    def mark_as_primary
      device.mark_as_primary!

      redirect_to devices_path
    end

    protected

    def skip_for_confirmed
      return unless device.is_confirmed?

      redirect_to(devices_path, flash: { info: t('two_factor_devices.user_devices.device_already_confirmed') })
    end

    def check_user
      redirect_to root_path unless current_user.present?
    end

    def device_params
      params.require("#{device_type}_device").permit(:value)
    end

    def device_type
      params[:device_type]
    end

    def device
      @device ||= current_user.two_factor_devices.find(params[:id])
    end

    def device_manager
      TwoFactorDevices.device_manager
    end

    def otp_error_message(otp_error)
      case otp_error
      when TwoFactorDevices::PhoneOtpError
        t('two_factor_devices.user_devices.failed_send_confirmation_phone_otp')
      when TwoFactorDevices::EmailOtpError
        t('two_factor_devices.user_devices.failed_send_confirmation_email_otp')
      else
        t('two_factor_devices.user_devices.failed_send_confirmation_otp')
      end
    end
  end
end
